require_relative "../../lib/pip"


describe Pip::Polygon do
  let(:poly) { Pip::Polygon.new(points) }
  describe "square" do
    let(:points) { [[0,0], [0,2], [2,2], [2,0]] }
    it "takes args" do
      expect(poly.points).to eq points
    end
    
    it "finds the vertical boundaries" do
      expect(poly.vertical_boundaries_on(1)).to eq [0,2]
    end
  
    it "finds if a point is within it" do
      expect(poly.contains?(1,1)).to be_true
      expect(poly.contains?(1,3)).to be_false
    end
  end
  
  describe "utah shape" do
    let(:points) { [[0,0], [4,0], [4,1], [2,1], [2,2], [0,2]] }
    
    it "has vertical boundaries" do
      expect(poly.vertical_boundaries_on(0)).to eq [0,4]
      expect(poly.vertical_boundaries_on(1)).to eq [0, 2, 4]
      expect(poly.vertical_boundaries_on(2)).to eq [0, 2]
      expect(poly.vertical_boundaries_on(3)).to eq []
    end
    
    it "finds if a point is within a polygon" do
      expect(poly.contains?(1,1)).to be_true
      expect(poly.contains?(1,1)).to be_true
      expect(poly.contains?(0,1)).to be_true
      expect(poly.contains?(0,0)).to be_true
      expect(poly.contains?(4,1)).to be_true
      expect(poly.contains?(4,2)).to be_false
      expect(poly.contains?(3,2)).to be_false
      expect(poly.contains?(999,999)).to be_false
    end
    
  end
  
  describe "triangle" do
    let(:points) { [
      [0,0],
      [3,0],
      [1,3]
    ] }
    
    it "finds the points within the polygon" do
      expect(poly.contains?(2,2)).to be_false
      expect(poly.contains?(0,1)).to be_false
      expect(poly.contains?(1,2)).to be_true
      expect(poly.contains?(2,1)).to be_true
      expect(poly.contains?(1,1)).to be_true
      expect(poly.contains?(1,0)).to be_true
      expect(poly.contains?(0,0)).to be_true
    end
    
    it "finds point equal to a point" do
      expect(poly.contains?(1,3)).to be_true
    end
  end
  describe "insane shape" do
    let(:points) { [
      [10,10],
      [15,15],
      [25,5],
      [30,20],
      [30,10],
      [40,30],
      [31,15],
      [31,25],
      [5,24],
      [10,10]
    ] }
    
    it "finds if a point is within a polygon" do
      expect(poly.contains?(10,10)).to be_true
      expect(poly.contains?(10,11)).to be_true
      expect(poly.contains?(25,6)).to be_true
      expect(poly.contains?(1,1)).to be_false
      expect(poly.contains?(40,30)).to be_true
      expect(poly.contains?(40,30)).to be_true
      expect(poly.contains?(40,31)).to be_false
    end
  end
  
  describe "lat longs" do
    let(:points) { [
      [47.606973,-122.341175],
      [47.610098,-122.333536],
      [47.60408,-122.328086],
      [47.601707,-122.336283]
    ] }
    
    it "finds if a point is within the longs" do
      expect(poly.contains?(47.601765,-122.325339)).to be_false
      expect(poly.contains?(47.603769,-122.324642)).to be_false
      expect(poly.contains?(47.606337,-122.334566)).to be_true
      expect(poly.contains?(47.60489,-122.338943)).to be_true
    end
  end
  
  describe "complicated lat longs" do
    let(:points) { [
      [47.606973,-122.341175],
      [47.605353,-122.33345],
      [47.610098,-122.333536],
      [47.601765,-122.325339],
      [47.601707,-122.336283]
    ] }
    
    it "finds if a point is withing the area" do
      expect(poly.contains?(47.606452,-122.331519)).to be_true
      expect(poly.contains?(47.608651,-122.336068)).to be_false
      expect(poly.contains?(47.606771,-122.334352)).to be_false
      expect(poly.contains?(47.605121,-122.333064)).to be_true
      expect(poly.contains?(47.609736,-122.332549)).to be_false
    end
  end
  
  describe "a super complicated lat long" do
    let(:points) { [
      [47.601736,-122.32079],
      [47.601794,-122.331991],
      [47.608333,-122.337999],
      [47.609693,-122.334652],
      [47.608796,-122.333665],
      [47.609606,-122.331562],
      [47.611255,-122.33109],
      [47.611342,-122.326927],
      [47.613541,-122.328944],
      [47.614062,-122.327399],
      [47.61409,-122.32079]
    ] }
    
    it "finds if a point is withing the area" do
      expect(poly.contains?(47.608709,-122.335639)).to be_true
      expect(poly.contains?(47.611024,-122.329888)).to be_true
      expect(poly.contains?(47.613483,-122.321649)).to be_true
      expect(poly.contains?(47.606279,-122.328215)).to be_true
      expect(poly.contains?(47.610272,-122.332678)).to be_false
      expect(poly.contains?(47.612007,-122.328429)).to be_false
      expect(poly.contains?(47.605035,-122.335854)).to be_false
      expect(poly.contains?(47.609288,-122.320747)).to be_false
      expect(poly.contains?(47.610619,-122.320876)).to be_true
    end
  end
end